use hence;
use itertools::Itertools;

#[derive(Debug)]
pub enum Node {
    Comment(String),
    List(Vec<Node>),
    String(String),
    Number(i32),
    Symbol(String),
    Literal(String),
}

impl hence::assembler::ToCode for Node {
    fn to_code(&self) -> String {
        match self {
            Node::Comment(x) => format!(";; {x}"),
            Node::List(x) => format!("({})", x.iter().map(|n| n.to_code()).join(" ")),
            Node::String(x) => format!("\"{x}\""),
            Node::Number(x) => x.to_string(),
            Node::Symbol(x) => format!(":{x}"),
            Node::Literal(x) => x.clone(),
        }
    }
}

#[derive(Debug)]
pub struct AST {
    pub body: Vec<Node>,
}
